
/** INCLUDES *******************************************************/
#include "mscext.h"
#include "USB/usb.h"
#include "USB/usb_function_generic.h"

#include "HardwareProfile.h"
#include "user.h"
#include "FreqMeter.h"

#include "Freq_DAC.h"

/** V A R I A B L E S ********************************************************/

#if defined(__18F2455) || defined(__18F2550) || defined(__18F4455) || defined(__18F4550)\
    || defined(__18F2458) || defined(__18F2453) || defined(__18F4558) || defined(__18F4553)
    #pragma udata USB_VARIABLES=0x500
#elif defined(__18F4450) || defined(__18F2450)
    #pragma udata USB_VARIABLES=0x480
#else
    #pragma udata
#endif

DATA_PACKET INPacket;
DATA_PACKET OUTPacket;
#pragma udata

USB_HANDLE USBGenericOutHandle = 0;
USB_HANDLE USBGenericInHandle = 0;

BOOL blinkStatusValid = TRUE;
BYTE counter;


/** P R I V A T E  P R O T O T Y P E S ***************************************/

void BlinkUSBStatus(void);
void ServiceRequests(void);

/** D E C L A R A T I O N S **************************************************/
#pragma code
void UserInit(void)
{
	INTCONbits.GIE = 0;
    RCONbits.IPEN = 0;			// No int priority
    INTCONbits.PEIE = 1;		// Peripheral INT enable

    mInitAllLEDs();
	InitFreqMeter();
	InitFreqDAC();
      
	StartFreqCount();

	INTCONbits.GIE = 1;			// enable global

    blinkStatusValid = TRUE;
}//end UserInit


// * Overview:        This function is a place holder for other user routines.
// *                  It is a mixture of both USB and non-USB tasks.
void ProcessIO(void)
{   
    //Blink the LEDs according to the USB device status
    if(blinkStatusValid)
    {
        BlinkUSBStatus();
    }

    // User Application USB tasks
    if((USBDeviceState < CONFIGURED_STATE)||(USBSuspendControl==1)) return;

    //respond to any USB commands that might have come over the bus
    ServiceRequests();

}//end ProcessIO


// * Overview:        This function takes in the commands from the PC from the
// *                  application and executes the commands requested
void ServiceRequests(void)
{
    BYTE index;
    
    //Check to see if data has arrived
    if(!USBHandleBusy(USBGenericOutHandle))
    {        
        //if the handle is no longer busy then the last
        //transmission is complete
       
        counter = 0;

        INPacket.CMD=OUTPacket.CMD;
        INPacket.Len=OUTPacket.Len;

        //process the command
        switch(OUTPacket.CMD)
        {
            case READ_VERSION:
                //dataPacket._byte[1] is len
                INPacket._byte[2] = MINOR_VERSION;
                INPacket._byte[3] = MAJOR_VERSION;
                counter=0x04;
                break;

			case READ_FREQ:
	        	for (index = 0; index < 4; index++)
	        		INPacket.Data[index] = MeasuredFreq.v[index];

				for (index = 0; index < 4; index++)
	        		INPacket.Data[index + 4] = AutoHoldFreq.v[index];
				for (index = 0; index < 4; index++)
	        		INPacket.Data[index + 8] = FreqError.v[index];
				for (index = 0; index < 2; index++)
	        		INPacket.Data[index + 12] = ActualFreq.v[index];

	        	INPacket.Len = 14;
	        	counter = 16;
				break;

			case AUTOHOLD:
				AutoHoldFreq.byte.LB = OUTPacket.Data[0];	// Set AutoHold Freq (0 means off)
				AutoHoldFreq.byte.HB = OUTPacket.Data[1];
				AutoHoldFreq.byte.UB = OUTPacket.Data[2];
				AutoHoldFreq.byte.MB = OUTPacket.Data[3];
				INPacket.Len = 4;
				counter = 6;								// Sends back the received data as ack
				break;
                                
            case RESET:
#ifdef __18CXX
                Reset();
#endif
                break;
                
            default:
#ifdef __18CXX
                Nop();
#endif
                break;
        }//end switch()
        if(counter != 0)
        {
            if(!USBHandleBusy(USBGenericInHandle))
            {
                USBGenericInHandle = USBGenWrite(USBGEN_EP_NUM,(BYTE*)&INPacket,counter);
            }
        }//end if
        
        //Re-arm the OUT endpoint for the next packet
        USBGenericOutHandle = USBGenRead(USBGEN_EP_NUM,(BYTE*)&OUTPacket,USBGEN_EP_SIZE);
    }//end if

}//end ServiceRequests


// * Overview:        BlinkUSBStatus turns on and off LEDs 
// *                  corresponding to the USB device state.
void BlinkUSBStatus(void)
{
    static WORD led_count=0;
    
    if(led_count == 0)led_count = 10000U;
    led_count--;

    #define mLED_Both_Off()         {mLED_1_Off();mLED_2_Off();}
    #define mLED_Both_On()          {mLED_1_On();mLED_2_On();}
    #define mLED_Only_1_On()        {mLED_1_On();mLED_2_Off();}
    #define mLED_Only_2_On()        {mLED_1_Off();mLED_2_On();}

    if(USBSuspendControl == 1)
    {
        if(led_count==0)
        {
            mLED_1_Toggle();
            if(mGetLED_1())
            {
                mLED_2_On();
            }
            else
            {
                mLED_2_Off();
            }
        }//end if
    }
    else
    {
        if(USBDeviceState == DETACHED_STATE)
        {
            mLED_Both_Off();
        }
        else if(USBDeviceState == ATTACHED_STATE)
        {
            mLED_Both_On();
        }
        else if(USBDeviceState == POWERED_STATE)
        {
            mLED_Only_1_On();
        }
        else if(USBDeviceState == DEFAULT_STATE)
        {
            mLED_Only_2_On();
        }
        else if(USBDeviceState == ADDRESS_STATE)
        {
            if(led_count == 0)
            {
                mLED_1_Toggle();
                mLED_2_Off();
            }//end if
        }
        else if(USBDeviceState == CONFIGURED_STATE)
        {
            if(led_count==0)
            {      
                mLED_1_Toggle();         
                if(mGetLED_1())
                {
                    mLED_2_Off();
                }
                else
                {
                    mLED_2_On();
                }
            }//end if
        }//end if(...)
    }//end if(UCONbits.SUSPND...)

}//end BlinkUSBStatus



/** EOF user.c ***************************************************************/
